{
  Autor       : Michael Springwald
  Erstellt am : Sonntag, 05.Juli.2009
    -----------------------------------------------------------------------------------
  01| Montag.Jul.09                     | Dienstag.Jul.09                             |
    -----------------------------------------------------------------------------------
  02| Donnerstag.Jul.09                 | Freitag.Jul.09                              |
    -----------------------------------------------------------------------------------
  03|                                   |                                             |
    -----------------------------------------------------------------------------------
  04|                                   |                                             |
    -----------------------------------------------------------------------------------
  05|                                   |                                             |
    -----------------------------------------------------------------------------------
}

unit uPL_IT;

{$mode objfpc}{$H+}

interface

uses
  Classes, SysUtils, Graphics, contnrs, Controls,
  LCLType, LCLIntf, LCLProc
  ;

type
  TPL_ITL = class;
  TPL_ITL_alignment = (
                         ag_Hori    = 0,
                         ag_Verti   = 1,
                         AG_OneLine = 2

                      ); // TPL_ITL_ViewStyle

  TPL_ITL_ChangeMod = (
                        LC_Add       = 0,
                        LC_Insert    = 1,
                        LC_Delete    = 2,
                        LC_Clear     = 3,
                        LC_ItemIndex = 4,
                        LC_Alignment = 5
                     ); // TPL_ITL_ChangeMod

  TPL_ITL_OnChange = procedure(const aMod:TPL_ITL_ChangeMod) of Object;
  TPL_OnScroll     = procedure(const aScroll:Integer; const aScrollDir:Boolean) of Object;

  { TPL_IT }
  TPL_IT = class
  private
    fData: TObject;
    fImageIndex: Integer;
    fIndex: Integer;
    fItemRect: TRect;
    fPicture: TPicture;
    fText: WideString;
    procedure SetImageIndex(const AValue: Integer);
    procedure SetText(const AValue: WideString);
  protected
  public
    ITL:TPL_ITL;
    ItemX,ItemWidth,ItemHeigh:Integer;
    constructor Create;
    destructor Destroy; override;
    property ItemRect:TRect read fItemRect write fItemRect;
    procedure Paint(var px, py:Integer); Overload;
    procedure Paint; Overload;

  published
    property Picture:TPicture read fPicture write fPicture;
    property ImageIndex:Integer read fImageIndex write SetImageIndex;
    property Text:WideString read fText write SetText;
    property Index:Integer read fIndex write fIndex;
    property Data:TObject read fData write fData;
  end; // TPL_IT

  { TPL_ITL }
  TPL_ITL = class
  private
    fAlignment: TPL_ITL_alignment;
    fDefaultItemHeight: Integer;
    fDefaultItemWidth: Integer;
    fImageList: TImageList;
    fItems:TObjectList;
    fOnChange: TPL_ITL_OnChange;
    fNoChangeEvent:Boolean;
    fonScroll: TPL_OnScroll;
    function GetCount: Integer;
    function GetItems(aIndex: Integer): TPL_IT;
    procedure SetAlignment(const AValue: TPL_ITL_alignment);
    procedure SetItemIndex(const AValue: Integer);
  protected
    procedure DoChange(const aMod:TPL_ITL_ChangeMod);
    procedure DoScroll(const aScroll:Integer; const aScrollDir:Boolean);
  public
    fItemIndex: Integer;

    Canvas:TCanvas;
    LW, ViewFirstIndex, ViewLastIndex, LineCount:Integer;
    ScrollRect:TRect;
    Init:Boolean;
    CaptionHeighSpacing:Integer;
    constructor Create(const aCanvas:TCanvas);
    destructor Destroy; override;

    function AddItem(const aText:WideString):TPL_IT;
    function InsertItem(const aText:WideString; const aIndex:Integer):TPL_IT;
    function DelItem(const aIndex:Integer):Boolean;
    function Clear:Boolean;
    function GetItemIndexBYXY(const aMx, aMY:Integer):Integer;
    function GetItemIndexBYXYIT(const aMx, aMY:Integer):TPL_IT;
    function GetRightIndex(const aIndex:Integer):Integer;
    function GetLeftIndex(const aIndex:Integer):Integer;
    function LineHeight:Integer;
    function GetLineCount:Integer;
    function InitItems:Boolean;
    procedure Key_Move_agOneLine(const aKey:word; aShift:TShiftState);
    procedure Key_Move_agVertikal(const aKey:word; aShift:TShiftState);
    procedure Key_Move_ag_Hori(const aKey:word; aShift:TShiftState);
    procedure BeginUpdate;
    procedure EndUpdate;
    procedure SetIndexes;

    property Count:Integer read GetCount;
    property Items[aIndex:Integer]:TPL_IT read GetItems; default;
    property onChange:TPL_ITL_OnChange read fOnChange write fOnChange;
    property onScroll:TPL_OnScroll read fonScroll write fonScroll;
    property ImageList:TImageList read fImageList write fImageList;
    property Alignment:TPL_ITL_alignment read fAlignment write SetAlignment;
    property ItemIndex:Integer read fItemIndex write SetItemIndex;

    property DefaultItemWidth:Integer read fDefaultItemWidth write fDefaultItemWidth;
    property DefaultItemHeight:Integer read fDefaultItemHeight write fDefaultItemHeight;
  published
  end; // TPL_ITL

implementation

{ TPL_IT }
procedure TPL_IT.SetText(const AValue: WideString);
begin
  if fText=AValue then exit;
  fText:=AValue;
end; // TPL_IT.SetText

procedure TPL_IT.SetImageIndex(const AValue: Integer);
begin
  if fImageIndex=AValue then exit;
  fImageIndex:=AValue;
end; // TPL_IT.SetImageIndex

constructor TPL_IT.Create;
begin
  inherited Create;
  ItemWidth:=-1;
  fImageIndex:=-1;
  fItemRect:=Rect(0,0,0,0);
  ITL:=nil; ItemX:=-1;
end; // TPL_IT.Create

destructor TPL_IT.Destroy;
begin
  inherited Destroy;
end; // TPL_IT.Destroy

procedure TPL_IT.Paint(var px,py:Integer);
var
  pw,ph,tpw,tPH,tx,sy,sx:Integer;
  r:int64;
  s:Boolean;
begin
//  if Assigned(ITL.ImageList) then begin
  pw:=ITL.ImageList.Width; ph:=ITL.ImageList.Height;
//  end;
  px:=ItemRect.Left; py:=ItemRect.Top;
  sy:=py-ITL.ScrollRect.Top;
  sx:=px-ITL.ScrollRect.Left;

  if ITL.ItemIndex = index then begin
     ITL.Canvas.Brush.Color:=clBlue;
     ITL.Canvas.Pen.Color:=clRed;
     ITL.Canvas.Font.Color:=clWhite;
     s:=False;
  end
  else begin
    ITL.Canvas.Brush.Color:=clWhite;
    ITL.Canvas.Pen.Color:=clBlack;
    ITL.Canvas.Font.Color:=clBlack;
    s:=True;
  end;
  tpw:=ITL.DefaultItemWidth;
  tPH:=ITL.DefaultItemHeight;
  r:=Round((tpw-pw) / 2); tx:=round((tpw-pw) / 2);

  case ITL.Alignment of
    ag_Verti: begin
      ITL.Canvas.Rectangle(px-1,sy-1,px+pw+tpw+1,sy+ph);
      ITL.ImageList.Draw(ITL.Canvas,px,sy,index,True);
      ITL.Canvas.TextOut(px+pw,sy+2,Text);
    end; // ag_Verti

    ag_Hori: begin
      ITL.Canvas.Rectangle(px,sy,px+ITL.DefaultItemWidth,sy+ITL.DefaultItemHeight);
      ITL.ImageList.Draw(ITL.Canvas,px+r,sy,index,True);
      ITL.Canvas.TextOut(px,sy+ph+2,Text);
    end; // ag_Hori;

    AG_OneLine: begin
      ITL.Canvas.Rectangle(sx,py,sx+ITL.DefaultItemWidth,py+ITL.DefaultItemHeight);
      ITL.ImageList.Draw(ITL.Canvas,sx+r,sy,index,True);
      ITL.Canvas.TextOut(sx,sy+ph+2,Text);
    end; // AG_OneLine;
  end; // case

//  if ITL.Alignment = ag_Verti then begin
//    ItemRect:=Rect(px,py,px+pw+tpw,py+ph);

//    py:=py+ph+tPH+5;
//  end
//  else begin
//    ItemRect:=Rect(px,py,px+tpw,py+tph);
//    ItemHeigh:=tph;
//    ITL.ImageList.Draw(ITL.Canvas,px+r,sy,index,True);
//    ITL.Canvas.TextOut(px,sy+ph+2,Text);
    //if px+itl.CaptionWidth <= ITL.Canvas.Width-itl.CaptionWidth then begin
//      px:=px+tpw+5
//    end
    //else begin
//      itl.LW:=ITL.Canvas.Width-tpw; px:=2;
//      if not itl.Init then begin
//        inc(itl.LineCount);
//      end;
//      py:=py+tPH+ITL.CaptionHeighSpacing;
//
//    end;
//  end;
end; // TPL_IT.Paint

procedure TPL_IT.Paint;
var
  x,y:Integer;
begin
  x:=-1; y:=-1;
  Paint(x,y);
end; // TPL_IT.Paint


{ TPL_ITL }

function TPL_ITL.GetCount: Integer;
begin
  result:=fItems.Count;
end; // TPL_ITL.GetCount

function TPL_ITL.GetItems(aIndex: Integer): TPL_IT;
begin
  result:=fItems[aIndex] as TPL_IT;
end; // TPL_ITL.GetItems

procedure TPL_ITL.SetAlignment(const AValue: TPL_ITL_alignment);
begin
  if fAlignment=AValue then exit;
  fAlignment:=AValue;
  DoChange(LC_Alignment);
end; // TPL_ITL.SetAlignment

procedure TPL_ITL.SetItemIndex(const AValue: Integer);
var
  oldItemIndex:Integer;
begin
  if fItemIndex=AValue then exit;
  oldItemIndex:=fItemIndex; fItemIndex:=-1;
  if oldItemIndex > -1 then items[oldItemIndex].Paint;
  fItemIndex:=AValue;
  if fItemIndex > -1 then items[fItemIndex].Paint;
end; // TPL_ITL.SetItemIndex

procedure TPL_ITL.DoChange(const aMod: TPL_ITL_ChangeMod);
begin
  if (not fNoChangeEvent) and (Assigned(onChange)) then onChange(aMod);
end; // TPL_ITL.DoChange

procedure TPL_ITL.DoScroll(const aScroll: Integer; const aScrollDir:Boolean);
begin
  if Assigned(onScroll) then onScroll(aScroll, aScrollDir);
end; // TPL_ITL.DoScroll

constructor TPL_ITL.Create(const aCanvas:TCanvas);
begin
  inherited Create;
  fNoChangeEvent:=False;
  CaptionHeighSpacing:=5;
  DefaultItemWidth:=108; DefaultItemHeight:=50;
  lw:=-1; LineCount:=0;
  fItems:=TObjectList.Create;
  fImageList:=nil;
  fAlignment:=ag_Hori;
  Canvas:=aCanvas;
  ScrollRect:=Rect(0,0,0,0);
  ViewFirstIndex:=-1; ViewLastIndex:=-1;
  Init:=False;
end; // TPL_ITL.Create

destructor TPL_ITL.Destroy;
begin
  FreeAndNil(fItems);
  inherited Destroy;
end; // TPL_ITL.Destroy

function TPL_ITL.AddItem(const aText: WideString): TPL_IT;
var
  aItem:TPL_IT;
begin
  aItem:=TPL_IT.Create;
  aItem.Text:=aText;
  aItem.ITL:=self;
  fItems.Add(aItem);
  result:=Items[Count-1];
  DoChange(LC_Add);
end; // TPL_ITL.AddItem

function TPL_ITL.InsertItem(const aText: WideString; const aIndex: Integer
  ): TPL_IT;
var
  aItem:TPL_IT;
begin
  aItem:=TPL_IT.Create;
  aItem.Text:=aText;
  aItem.ITL:=self;
  fItems.Insert(aIndex,aItem);
  result:=Items[aIndex+1];
  DoChange(LC_Insert);
end; // TPL_ITL.InsertItem

function TPL_ITL.DelItem(const aIndex: Integer): Boolean;
begin
  if aIndex <=Count-1 then begin
    fItems.Delete(aIndex);
    DoChange(LC_Delete);
    result:=True;
  end
  else
    Result:=False;
end; // TPL_ITL.DelItem

function TPL_ITL.Clear: Boolean;
begin
  if fItems.Count > 0 then begin
    result:=True;
    fItems.Clear;
    DoChange(LC_Clear);
  end
  else
    result:=False;
end; // TPL_ITL.Clear

function TPL_ITL.GetItemIndexBYXY(const aMx, aMY:Integer): Integer;
var
  i,z,t:Integer;
  pt:TPoint;
  R:TRect;
begin
  pt:=Point(aMx, aMY); z:=-1;

  for i := ViewFirstIndex to ViewLastIndex do begin
    r.Top:=items[i].ItemRect.Top-ScrollRect.Top;
    r.Bottom:=r.Top+Items[i].ItemHeigh;
    r.Left:=items[i].ItemRect.Left-ScrollRect.Left;
    r.Right:=r.Left+Items[i].ItemWidth;
    if PtInRect(R,pt) then begin
//      writeln(items[i].text,'\',items[i].Index);
      z:=i;
      break;
    end;
  end; // for i
  result:=z;
end; // TPL_ITL.GetItemIndexBYXY

function TPL_ITL.GetItemIndexBYXYIT(const aMx, aMY:Integer): TPL_IT;
var
  index:Integer;
begin
  index:=GetItemIndexBYXY(aMX, aMY);
  if index > -1 then result:=items[index] else result:=nil;
end; // TPL_ITL.GetItemIndexBYXYIT

procedure TPL_ITL.SetIndexes;
var
  i:Integer;
begin
  for i := 0 to Count-1 do begin
    Items[i].Index:=i;
  end;
end; // TPL_ITL.SetIndexes

function TPL_ITL.GetRightIndex(const aIndex:Integer): Integer;
var
  i,z:integer;
begin
  z:=-1;
  for i:=aindex +1 to Count-1 do begin
    if (Items[i].ItemRect.Top > Items[aIndex].ItemRect.Top) then begin
      if Items[i].ItemRect.Left = Items[aIndex].ItemRect.Left then begin
        z:=i;
        break;
      end;
    end;
  end; // for i
  result:=z;
end; // TPL_ITL.GetRightIndex

function TPL_ITL.GetLeftIndex(const aIndex: Integer): Integer;
var
  i,z,m:integer;
begin
  z:=-1;
  if aIndex = -1 then begin result:=-1; exit; end;
  for i:=aIndex-1 downto 0 do begin
    if (Items[i].ItemRect.Top < Items[aIndex].ItemRect.Top) then begin
      if (items[i].ItemRect.Left <=Items[aIndex].ItemRect.Left) then begin
        z:=i;
        break;
      end;
    end;
  end; // for i
  result:=z;
end; // TPL_ITL.GetLeftIndex

function TPL_ITL.LineHeight: Integer;
begin

end; // TPL_ITL.LineHeight

function TPL_ITL.GetLineCount: Integer;
{var
  cx:Integer;
begin
  if ScrollRect.Right > 0 then begin
    cx:=(ScrollRect.Right div (CaptionWidth+5))-1;
    writeln('CX:',cx,'\',ScrollRect.Right);
    result:=(Count) div cx;
  end;}

var
  i,z:Integer;
begin
  z:=0;
  for i:=0 to Count-1 do begin
    if items[i].ItemRect.Left = 2 then begin
      writeln('text:',items[i].text);
      inc(z);
    end;
  end;
  result:=z;
end; // TPL_ITL.GetLineCount

function TPL_ITL.InitItems: Boolean;
var
  i,px,py,ph,pw:Integer;
  ok:Boolean;
begin
  px:=2; py:=5; pw:=DefaultItemWidth; ph:=DefaultItemHeight; ok:=False;
  for i:=0 to Count -1 do begin
    with items[i] do begin
      case Alignment of
        ag_Hori: begin
          ItemRect:=Rect(px,py,px+pw,py+ph);
          ItemWidth:=pw+5;
          ItemHeigh:=ph+5;
          if px+pw+5 <=ScrollRect.Right-pw-5 then
            px:=px+pw+5
          else begin
            px:=2;
            inc(py,ph+5);
          end;
        end; // ag_Hori;
        ag_Verti: begin
          ItemRect:=Rect(px,py,px+pw,py+ph);
          ItemWidth:=pw+5;
          ItemHeigh:=ph;
          py:=py+ph+5;
        end; // ag_Verti;
        AG_OneLine: begin
          ItemRect:=Rect(px,py,px+pw,py+ph);
          ItemWidth:=pw+5;
          ItemHeigh:=ph;
          px:=px+pw+5;
        end; // AG_OneLine
      end; // with
    end; // case
  end; // for i
  ok:=True;
  result:=OK;
end; // TPL_ITL.InitItems

procedure TPL_ITL.Key_Move_agOneLine(const aKey: word; aShift: TShiftState);
var
  i,sx:Integer;
begin
  case akey of
    VK_LEFT: begin
      if ItemIndex-1 >= 0 then begin
        i:=ItemIndex-1;
        if items[i].ItemRect.Left <=ScrollRect.Left then begin
          DoScroll(SB_LINELEFT,True);
        end;
        ItemIndex:=i;
      end
      else begin
        DoScroll(SB_RIGHT,True);
        ItemIndex:=Count-1;
      end;
    end; // VK_LEFT

    VK_RIGHT: begin
      if ItemIndex+1 <= Count-1 then begin
        i:=ItemIndex+1;
        sx:=items[i].ItemRect.Left-ScrollRect.Left;
        if sx >=ScrollRect.Right then begin
          DoScroll(SB_LINERIGHT,True);
        end;
        ItemIndex:=i;
      end
      else begin
        DoScroll(SB_LEFT,True);
        ItemIndex:=0;
      end;
    end; // VK_RIGHT
  end; // case
end; // TPL_ITL.Key_Move_agOneLine

procedure TPL_ITL.Key_Move_agVertikal(const aKey: word; aShift: TShiftState);
var
  i:Integer;
begin
  case aKey of
    VK_UP: begin
      if ItemIndex-1 >=0 then begin
        i:=ItemIndex-1;
        if Items[i].ItemRect.Top <=ScrollRect.Top then
          DoScroll(SB_LINEUP,False);
        ItemIndex:=i;
      end
      else begin
        ItemIndex:=Count-1;
        DoScroll(SB_BOTTOM,False);
      end;
    end; // VK_UP

    VK_DOWN: begin
      if ItemIndex+1 <=Count-1 then begin
        i:=ItemIndex+1;
        if Items[i].ItemRect.Top >= (ScrollRect.Top+ScrollRect.Bottom)-items[i].ItemHeigh then
          DoScroll(SB_LINEDOWN,False);
        ItemIndex:=i;
      end
      else begin
        ItemIndex:=0;
        DoScroll(SB_TOP,False);
      end;
    end; // VK_DOWN
  end; // case key
end; // TPL_ITL.Key_Move_agVertikal

procedure TPL_ITL.Key_Move_ag_Hori(const aKey: word; aShift: TShiftState);
var
  i,sx:Integer;
begin
  case aKey of
    VK_LEFT: begin
      if ItemIndex-1 >=0 then begin
        i:=ItemIndex-1;
        if Items[i].ItemRect.Top <=ScrollRect.Top then
          DoScroll(SB_LINEUP,False);
        ItemIndex:=i;
      end
      else begin
        ItemIndex:=Count-1;
        DoScroll(SB_BOTTOM,False);
      end;
    end; // VK_LEFT

    VK_RIGHT: begin
      if ItemIndex+1 <=Count-1 then begin
        i:=ItemIndex+1;
        if Items[i].ItemRect.Top > (ScrollRect.Top+ScrollRect.Bottom)-items[i].ItemHeigh then
          DoScroll(SB_LINEDOWN,False);
        ItemIndex:=i;
      end
      else begin
        ItemIndex:=0;
        DoScroll(SB_TOP,False);
      end;
    end; // VK_RIGHT

     VK_DOWN: begin
       if (ItemIndex = Count-1) or (ItemIndex = -1) then begin
         ItemIndex:=0;
         DoScroll(SB_TOP,False);
       end
       else begin
         i:=GetRightIndex(ItemIndex);
         if (i <=Count-1) and (i > -1) then begin
           if Items[i].ItemRect.Top > (ScrollRect.Top+ScrollRect.Bottom)-items[i].ItemHeigh then
             DoScroll(SB_LINEDOWN,False);
           ItemIndex:=i;
         end;
       end;
     end; // VK_DOWN

     VK_UP: begin
       if ItemIndex > 0 then begin
         i:=GetLeftIndex(ItemIndex);
         if i >=0 then begin
           if Items[i].ItemRect.Top <=ScrollRect.Top then
             DoScroll(SB_LINEUP,False);
           ItemIndex:=i;
         end;
       end
       else begin
         ItemIndex:=Count-1;
         DoScroll(SB_BOTTOM,False);
       end;
     end; // VK_UP;
  end; // case
end; // TPL_ITL.Key_Move_ag_Hori

procedure TPL_ITL.BeginUpdate;
begin
  fNoChangeEvent:=True;
end; // TPL_ITL.BeginUpdate

procedure TPL_ITL.EndUpdate;
begin
  fNoChangeEvent:=False;
end; // TPL_ITL.EndUpdate

end.

